/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.ArrayList;


public class Net {
    protected String version;
	protected Declaration[] declarations;
	protected Fc2Table[] globalTables;
	protected Label globalLabel;
	protected NetNode[] nets;

    public Net(String version, Declaration[] d, Fc2Table[] gtables,
        Label glabel, NetNode[] n) {
        this(d, gtables, glabel, n);
        this.version = version;
    }

    public Net(Declaration[] d, Fc2Table[] gtables, Label glabel, NetNode[] n) {
        declarations = d;
        globalTables = gtables;
        globalLabel = glabel;
        nets = n;
    }

    public Net(String version, ArrayList ds, ArrayList gtables, Label glabel,
        NetNode[] n) {
        this(ds, gtables, glabel, n);
        this.version = version;
    }

    public Net(ArrayList ds, ArrayList gtables, Label glabel, NetNode[] n) {
        declarations = (Declaration[]) ds.toArray(new Declaration[0]);
        globalTables = (Fc2Table[]) gtables.toArray(new Fc2Table[0]);
        globalLabel = glabel;
        nets = n;
    }

    public Net() {
        declarations = new Declaration[0];
        globalTables = new Fc2Table[0];
        globalLabel = new Label();
        nets = new NetNode[0];
    }

    public Declaration[] get_declarations() {
        return declarations;
    }

    public Fc2Table[] get_globalTables() {
        return globalTables;
    }

    public void set_tables(Fc2Table[] gtables) {
        globalTables = gtables;
    }

    public Label get_globalLabel() {
        return globalLabel;
    }

    public void set_label(Label glabel) {
        globalLabel = glabel;
    }

    public NetNode[] get_nets() {
        return nets;
    }

    public void set_nets(NetNode[] n) {
        nets = n;
    }

    public void pp(java.io.PrintStream out, boolean s) {
        out.println("version " + getVersion());

        if (declarations != null && 0 < declarations.length) {
            out.println(Types.getTypeName(Types.LIST_DECLARATIONS, s));

            for (int i = 0; i < declarations.length; i++) {
                declarations[i].pp(out, s, getVersion());
            }
        }

        out.println(Types.getTypeName(Types.TABLE_NETS, s) + nets.length);

        for (int i = 0; i < globalTables.length; i++) {
            globalTables[i].pp(out, s);
        }

        if (globalLabel != null) {
            globalLabel.pp(out, s);
        }

        for (int i = nets.length-1; i >= 0; i--) {
            out.println(Types.getTypeName(Types.TYPE_NET, s) + i);
            nets[i].pp(out, s);
        }
    }

public Types getSymbolTable() {
	Types simbolTable = new Types();
	Declaration[] decl = get_declarations();

	for (int i = 0; i < decl.length; i++) {
		if (decl[i].getCodomain() == Types.TYPE_ERROR) {
			Utils.reportError("Bad return type in " +
				Types.getMethodName(decl[i].getClasse()) + " " +
				decl[i].getToken());
		}

		boolean errorArg = false;
		boolean errorRet = false;

		for (int j = 0; j < decl[i].getDomain().length; j++) {
			if ((decl[i].getDomain()[j] == Types.TYPE_ERROR) && !errorArg) {
				Utils.reportError("Bad argument types in " +
					Types.getMethodName(decl[i].getClasse()) + " " +
					decl[i].getToken());
				errorArg = true;
			}

			if ((decl[i].getDomain()[j] == Types.TYPE_ANY) &&
					(decl[i].getCodomain() != Types.TYPE_ANY) && !errorRet) {
				Utils.reportError("The return type of the  " +
					Types.getMethodName(decl[i].getClasse()) + " " +
					decl[i].getToken() + " must be " +
					Types.getTypeName(Types.TYPE_ANY));
				errorRet = true;
			}

			if (errorArg && errorRet) {
				break;
			}
		}

		simbolTable.addType(decl[i]);
	}
	return (simbolTable);
}

public boolean verifyTypes() {
	return(verifyTypes(getSymbolTable()));
}

public boolean verifyTypes(Types simbolTable) {
	boolean result = true;
	simbolTable.addDefaultTypes();
	
	TypeCheckVisitor checkVisitor = new TypeCheckVisitor(simbolTable);


	// rule T1
	result = get_globalLabel().verifyType(checkVisitor);

	// rule T2
	for (int i = 0; i < get_globalTables().length; i++)
		result = get_globalTables()[i].verifyType(checkVisitor);

	NetNode[] nets = get_nets();

	for (int i = 0; i < nets.length; i++) {
		result = result && nets[i].getLabel().verifyType(checkVisitor);

		// rule T2
		for (int j = 0; j < nets[i].getLocalTables().length; j++)
			result = nets[i].getLocalTables()[j].verifyType(checkVisitor);

		// begin rule T1
		nets[i].getLabel().verifyType(checkVisitor);

		Vertex[] vertices = nets[i].getVertice();

		for (int j = 0; j < vertices.length; j++) {
			vertices[j].getLabel().verifyType(checkVisitor);

			Edge[] edges = vertices[j].getEdges();

			for (int k = 0; k < edges.length; k++)
				edges[k].getLabel().verifyType(checkVisitor);
		}

		// end rule T1

		/*
					// Exception E3
					Vector hooks = nets[i].getLabel().get_hooks();
					boolean trueNet = false;

					for (Enumeration e = hooks.elements(); e.hasMoreElements();) {
						Object elm = e.nextElement();
						if ("\"synch_vector\"".equals(elm.toString()) ||
								"\"transducer\"".equals(elm.toString())) {
							trueNet = true;
						}
					}

					if (trueNet) {
						// Exception E3a
						Vertex[] vertices = nets[i].getVertice();

						for (int j = 0; j < vertices.length; j++) {
							if ((vertices[j].getLabel().get_struc() != null) &&
									vertices[j].getLabel().get_struc() instanceof OpExpr &&
									((OpExpr) vertices[j].getLabel().get_struc()).getOp()
										 .equals("<")) {
								result = result &&
									((OpExpr) vertices[j].getLabel().get_struc()).getSons()[1].verifyType(simbolTable,
										Types.TYPE_NET);

								;
							}

							if ((vertices[j].getLabel().get_behav() != null) &&
									vertices[j].getLabel().get_behav() instanceof OpExpr &&
									((OpExpr) vertices[j].getLabel().get_behav()).getOp()
										 .equals("<")) {
								// Exception E3c
								if (!((OpExpr) vertices[j].getLabel().get_behav()).getSons()[1].verifyType(
											simbolTable, Types.TYPE_VERTEX)) {
									Utils.reportError("Expresion " +
										vertices[j].getLabel().get_behav().toString() +
										" did not meet the exception rule E3c, see the FC2: Reference Manual");
									result = false;
								}
							}

							Edge[] edges = vertices[j].getEdges();

							for (int k = 0; k < edges.length; k++) {
								// Exception E3b
								if ((edges[k].getLabel().get_struc() != null) &&
										edges[k].getLabel().get_struc() instanceof OpExpr &&
										((OpExpr) edges[k].getLabel().get_struc()).getOp()
											 .equals("<")) {
									if (!((OpExpr) edges[k].getLabel().get_struc()).getSons()[1].verifyType(
												simbolTable, Types.TYPE_VERTEX)) {
										Utils.reportError("Expresion " +
											edges[k].getLabel().get_struc().toString() +
											" did not meet the exception rule E3b, see the FC2: Reference Manual");
										result = false;
									}
								}

								if ((edges[k].getLabel().get_behav() != null) &&
										edges[k].getLabel().get_behav() instanceof OpExpr &&
										((OpExpr) edges[k].getLabel().get_behav()).getOp()
											 .equals("<")) {
									// Exception E3c
									if (!((OpExpr) edges[k].getLabel().get_behav()).getSons()[1].verifyType(
												simbolTable, Types.TYPE_VERTEX)) {
										Utils.reportError("Expresion " +
											edges[k].getLabel().get_behav().toString() +
											" did not meet the exception rule E3c, see the FC2: Reference Manual");
										result = false;
									}
								}
							}
						}
					}
					*/
	}

	return (result);
}


    /**
     * Returns the version.
     * @return String
     */
    public String getVersion() {
        return version;
    }
	/**
	 * @return Declaration[]
	 */
	public Declaration[] getDeclarations() {
		return declarations;
	}

	/**
	 * @return Label
	 */
	public Label getGlobalLabel() {
		return globalLabel;
	}

	/**
	 * @return Fc2Table[]
	 */
	public Fc2Table[] getGlobalTables() {
		return globalTables;
	}

	/**
	 * @return NetNode[]
	 */
	public NetNode[] getNets() {
		return nets;
	}

	/**
	 * Sets the declarations.
	 * @param declarations The declarations to set
	 */
	public void setDeclarations(Declaration[] declarations) {
		this.declarations = declarations;
	}

	/**
	 * Sets the globalLabel.
	 * @param globalLabel The globalLabel to set
	 */
	public void setGlobalLabel(Label globalLabel) {
		this.globalLabel = globalLabel;
	}

	/**
	 * Sets the globalTables.
	 * @param globalTables The globalTables to set
	 */
	public void setGlobalTables(Fc2Table[] globalTables) {
		this.globalTables = globalTables;
	}

	/**
	 * Sets the nets.
	 * @param nets The nets to set
	 */
	public void setNets(NetNode[] nets) {
		this.nets = nets;
	}

	/**
	 * Sets the version.
	 * @param version The version to set
	 */
	public void setVersion(String version) {
		this.version = version;
	}

}
