/* 
* ################################################################
* 
* FC2Parser: A Java(TM) parser for FC2 Format files
* 
* Copyright (C) 2004 INRIA
* Contact: vercors@sophia.inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               Oasis Project
*                        http://www.inria.fr/oasis
*  Contributor(s): 
* 
* ################################################################
*/ 
package fr.inria.oasis.fc2.parser;

import java.util.ArrayList;


public class OpExpr extends Expr {
    public static final int HIGH_PRIORITY = 0;
    public static final int LOW_PRIORITY = 60;
    public String op;
    public Expr[] sons;
    protected int priority;
    protected int type;

    public OpExpr(String s, Expr[] ss, int type, int priority) {
        op = s;
        sons = ss;
        this.type = type;
        this.priority = priority;
    }

    public OpExpr(String s, Expr[] ss, int type) {
        this(s, ss, type, OpExpr.LOW_PRIORITY);
    }

    public int hashCode() {
        int hashCode = 1;

        for (int i = 0; i < sons.length; i++) {
            Object obj = sons[i];
            hashCode = (hashCode*31) + ((obj == null) ? 0 : obj.hashCode());
        }
        
        return (hashCode+getOp().hashCode());
    }

    public Object clone() {
        Expr[] newSons = new Expr[sons.length];

        for (int i = 0; i < newSons.length; i++)
            newSons[i] = (Expr) sons[i].clone();

        return (new OpExpr(new String(op), newSons, type, priority));
    }

    public boolean equals(Object o) {
        if (o == null) {
            return (false);
        }

        try {
            OpExpr expr = (OpExpr) o;

            if (((op == null) ? (expr.getOp() == null) : op.equals(expr.getOp())) &&
                    (priority == priority) && (type == type) &&
                    (sons.length == expr.getSons().length)) {
                for (int i = 0; i < sons.length; i++) {
                    if (!sons[i].equals(expr.getSons()[i])) {
                        return (false);
                    }
                }

                return (true);
            }

            return (false);
        } catch (ClassCastException e) {
            return (false);
        }
    }

    ArrayList decode_targets() throws Exception {
        if ("+" == op) {
            ArrayList l = sons[0].decode_targets();
            l.addAll(sons[1].decode_targets());

            return l;
        } else {
            throw new Exception("OpExpr: bad targets: " + op);
        }
    }

    public String toString() {
        switch (getType()) {
        case Types.OPERATOR_CONSTANT:
            return (String) op;

        case Types.OPERATOR_UNARY:
            return (op + parens(sons[0]));

        case Types.OPERATOR_INFIX:
            return (parens(sons[0]) + " " + op + " " + parens(sons[1]));

        default: {
            String res = op + "(";

            for (int i = 0; i < (sons.length - 1); i++) {
                res = res + sons[i].toString() + ",";
            }

            // case when sons are 0
            if (sons.length == 0) {
                res = res + ")";
            } else {
                res = res + sons[sons.length - 1].toString() + ")";
            }

            return res;
        }
        }
    }

    //	private String parens(Expr son) {
    // Exprs are allways lower than OpExprs.
    //		return son.toString();
    //	}
    private String parens(Expr son) {
        if (!(son instanceof OpExpr)) {
            return son.toString();
        }

        OpExpr sonop = (OpExpr) son;

        if ((sonop.getPriority() < getPriority()) ||
                (sonop.getType() != Types.OPERATOR_INFIX)) {
            return son.toString();
        }

        if (this.op.equals(sonop.op)) { // all infixes are left associative

            return son.toString(); // so we don't need parens
        } else { // different ops, we need parens.

            return ("(" + son.toString() + ")");
        }
    }

    /**
     * Returns the priority.
     * @return int
     */
    public int getPriority() {
        return priority;
    }

    /**
     * Returns the type.
     * @return int
     */
    public int getType() {
        return type;
    }

    /**
     * Sets the priority.
     * @param priority The priority to set
     */
    public void setPriority(int priority) {
        this.priority = priority;
    }

    /**
     * Sets the type.
     * @param type The type to set
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * Returns the op.
     * @return String
     */
    public String getOp() {
        return op;
    }

    /**
     * Returns the sons.
     * @return Expr[]
     */
    public Expr[] getSons() {
        return sons;
    }

    /**
     * Sets the op.
     * @param op The op to set
     */
    public void setOp(String op) {
        this.op = op;
    }

    /**
     * Sets the sons.
     * @param sons The sons to set
     */
    public void setSons(Expr[] sons) {
        this.sons = sons;
    }

    protected int getParameterSize() {
        if (!getOp().equals(",")) {
            return (1);
        }

        int total = 0;

        for (int i = 0; i < getSons().length; i++)
            total += ((OpExpr) getSons()[i]).getParameterSize();

        return total;
    }

    public Object dispatch(Visitor visitor, Object[] params) {
        return (visitor.visit(this, params));
    }
}
